from kopeechka.async_methods import AsyncMethods
from kopeechka.config import sft_id
from kopeechka.kopeechka_types import UserBalance, GetEmail, GetMessage, Status, GetTaskId, Domains, MailboxZones, Zone, Popular
from kopeechka.error import catch_error


class AsyncMailActivations(AsyncMethods):
    def __init__(self, api_token: str, timeout: int = 5):
        super().__init__(api_token, timeout)

    async def user_balance(self,
                           **kwargs
                           ) -> UserBalance:
        json_data = await super().user_balance(**kwargs)
        catch_error(json_data)
        return UserBalance(**json_data)

    async def mailbox_get_email(self,
                                site: str,
                                mail_type: str = "",
                                regex: str = "",
                                soft_id: int = sft_id,
                                investor: int = "",
                                subject: str = "",
                                password: int = 0,
                                **kwargs
                                ) -> GetEmail:
        json_data = await super().mailbox_get_email(site,
                                                    mail_type,
                                                    regex,
                                                    soft_id,
                                                    investor,
                                                    subject,
                                                    password,
                                                    **kwargs
                                                    )
        catch_error(json_data)
        return GetEmail(**json_data)

    async def mailbox_get_message(self,
                                  task_id: int,
                                  full: int = "",
                                  **kwargs
                                  ) -> GetMessage:
        json_data = await super().mailbox_get_message(task_id, full, **kwargs)
        catch_error(json_data)
        return GetMessage(**json_data)

    async def mailbox_cancel(self,
                             task_id: int,
                             **kwargs
                             ) -> Status:
        json_data = await super().mailbox_cancel(task_id, **kwargs)
        catch_error(json_data)
        return Status(**json_data)

    async def mailbox_reorder(self,
                              site: str,
                              email: str,
                              regex: str = "",
                              subject: str = "",
                              password: int = "",
                              **kwargs
                              ) -> GetEmail:
        json_data = await super().mailbox_reorder(site, email, regex, subject, password, **kwargs)
        catch_error(json_data)
        return GetEmail(**json_data)

    async def mailbox_get_fresh_id(self,
                                   site: str,
                                   email: str,
                                   **kwargs
                                   ) -> GetTaskId:
        json_data = await super().mailbox_get_fresh_id(site, email, **kwargs)
        catch_error(json_data)
        return GetTaskId(**json_data)

    async def mailbox_get_domains(self,
                                  **kwargs
                                  ) -> Domains:
        json_data = await super().mailbox_get_domains(**kwargs)
        catch_error(json_data)
        return Domains(**json_data)

    async def mailbox_zones(self,
                            popular: int = "",
                            zones: int = "",
                            site: str = "",
                            **kwargs
                            ) -> MailboxZones:
        json_data = await super().mailbox_zones(popular, zones, site, **kwargs)
        catch_error(json_data)
        return MailboxZones(json_data.get("status"), zones=[Zone(**x) for x in json_data.get("zones")] if json_data.get("zones") else None, popular=[Popular(**x) for x in json_data.get("popular")] if json_data.get("popular") else None)



from kopeechka.methods import Methods
from kopeechka.async_methods import AsyncMethods
from kopeechka.kopeechka_types import UserBalance, GetEmail, GetMessage, GetTaskId, Domains, Zone, Popular, Status, MailboxZones
from kopeechka.mail_activations import MailActivations
from kopeechka.async_mail_activations import AsyncMailActivations
from kopeechka.error import KopeechkaApiError, TimeOut

__all__ = (
    "Methods",
    "AsyncMethods",
    "UserBalance",
    "GetEmail",
    "GetMessage",
    "GetTaskId",
    "Domains",
    "Zone",
    "Popular",
    "Status",
    "MailboxZones",
    "MailActivations",
    "AsyncMailActivations",
    "KopeechkaApiError",
    "TimeOut",
)



from dataclasses import dataclass

from typing import List


@dataclass
class UserBalance:
    status: str
    balance: float


@dataclass
class GetEmail:
    status: str
    id: int
    mail: str


@dataclass
class GetMessage:
    status: str
    value: str
    fullmessage: str


@dataclass
class Status:
    status: str


@dataclass
class GetTaskId:
    status: str
    id: int


@dataclass
class Domains:
    status: str
    count: int
    domains: List[str]


@dataclass
class Zone:
    name: str
    cost: int


@dataclass
class Popular:
    name: str
    cost: int
    count: int


@dataclass
class MailboxZones:
    status: str
    zones: List[Zone] = None
    popular: List[Popular] = None



from aiohttp import ClientSession, ClientTimeout
from kopeechka.config import sft_id
import asyncio


class AsyncMethods:
    def __init__(self, api_token: str, timeout: int = 5):
        self.api_token: str = api_token
        self.default_query = {
            "token": self.api_token,
            "type": "JSON",
            "api": 2.0
        }
        self.timeout = ClientTimeout(total=timeout)
        self.valid_token()

    def valid_token(self):
        asyncio.run(self.user_balance())

    async def request(self, url: str = "", params: dict = None):
        params.update(self.default_query)

        async with ClientSession(timeout=self.timeout) as session:
            response = await session.get(url=url, params=params)
        return response

    async def user_balance(self, **kwargs) -> dict:
        query = {
            **kwargs
        }
        response = await self.request("http://api.kopeechka.store/user-balance", params=query)
        return await response.json()

    async def mailbox_get_email(self, site: str, mail_type: str = "", regex: str = "",
                                soft_id: int = sft_id, investor: int = "", subject: str = "", password: int = 0,
                                **kwargs) -> dict:
        query = {
            "site": site,
            "mail_type": mail_type,
            "regex": regex,
            "soft": soft_id,
            "investor": investor,
            "subject": subject,
            "password": password,
            **kwargs
        }

        response = await self.request("http://api.kopeechka.store/mailbox-get-email", params=query)
        return await response.json()

    async def mailbox_get_message(self, task_id: int, full: int = "", **kwargs) -> dict:
        query = {
            "id": task_id,
            "full": full,
            **kwargs
        }

        response = await self.request("http://api.kopeechka.store/mailbox-get-message", params=query)
        return await response.json()

    async def mailbox_cancel(self, task_id: int, **kwargs) -> dict:
        query = {
            "id": task_id,
            **kwargs
        }

        response = await self.request("http://api.kopeechka.store/mailbox-cancel", params=query)
        return await response.json()

    async def mailbox_reorder(self, site: str, email: str, regex: str = "", subject: str = "", password: int = "",
                              **kwargs) -> dict:
        query = {
            "site": site,
            "email": email,
            "regex": regex,
            "subject": subject,
            "password": password,
            **kwargs
        }

        response = await self.request("http://api.kopeechka.store/mailbox-reorder", params=query)
        return await response.json()

    async def mailbox_get_fresh_id(self, site: str, email: str, **kwargs) -> dict:
        query = {
            "site": site,
            "email": email,
            **kwargs
        }

        response = await self.request("http://api.kopeechka.store/mailbox-get-fresh-id", params=query)
        return await response.json()

    async def mailbox_get_domains(self, **kwargs) -> dict:
        query = {
            **kwargs
        }

        response = await self.request("http://api.kopeechka.store/mailbox-get-domains", params=query)
        return await response.json()

    async def mailbox_zones(self, popular: int = "", zones: int = "", site: str = "", **kwargs) -> dict:
        query = {
            "popular": popular,
            "zones": zones,
            "site": site,
            **kwargs
        }

        response = await self.request("https://api.kopeechka.store/mailbox-zones", params=query)
        return await response.json()



from setuptools import setup

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

if __name__ == '__main__':
    setup(
        name='kopeechka',
        version='8.0',
        description='This code is a representation of the kopeechka.store API in Python',
        project_urls={
            "Source": "https://github.com/livvyyRUS/kopeechka",
        },
        long_description=long_description,
        long_description_content_type='text/markdown',
        packages=['kopeechka'],
        install_requires=["requests", "aiohttp"],
        author_email='rem.game.on@gmail.com'
    )



from kopeechka.methods import Methods
from kopeechka.config import sft_id
from kopeechka.kopeechka_types import UserBalance, GetEmail, GetMessage, Status, GetTaskId, Domains, MailboxZones, Zone, Popular
from kopeechka.error import catch_error


class MailActivations(Methods):
    def __init__(self, api_token: str, timeout: int = 5):
        super().__init__(api_token, timeout)

    def user_balance(self,
                     **kwargs
                     ) -> UserBalance:
        json_data = super().user_balance(**kwargs)
        catch_error(json_data)
        return UserBalance(**json_data)

    def mailbox_get_email(self,
                          site: str,
                          mail_type: str = "",
                          regex: str = "",
                          soft_id: int = sft_id,
                          investor: int = "",
                          subject: str = "",
                          password: int = 0,
                          **kwargs
                          ) -> GetEmail:
        json_data = super().mailbox_get_email(site,
                                              mail_type,
                                              regex,
                                              soft_id,
                                              investor,
                                              subject,
                                              password,
                                              **kwargs
                                              )
        catch_error(json_data)
        return GetEmail(**json_data)

    def mailbox_get_message(self,
                            task_id: int,
                            full: int = "",
                            **kwargs
                            ) -> GetMessage:
        json_data = super().mailbox_get_message(task_id, full, **kwargs)
        catch_error(json_data)
        return GetMessage(**json_data)

    def mailbox_cancel(self,
                       task_id: int,
                       **kwargs
                       ) -> Status:
        json_data = super().mailbox_cancel(task_id, **kwargs)
        catch_error(json_data)
        return Status(**json_data)

    def mailbox_reorder(self,
                        site: str,
                        email: str,
                        regex: str = "",
                        subject: str = "",
                        password: int = "",
                        **kwargs
                        ) -> GetEmail:
        json_data = super().mailbox_reorder(site, email, regex, subject, password, **kwargs)
        catch_error(json_data)
        return GetEmail(**json_data)

    def mailbox_get_fresh_id(self,
                             site: str,
                             email: str,
                             **kwargs
                             ) -> GetTaskId:
        json_data = super().mailbox_get_fresh_id(site, email, **kwargs)
        catch_error(json_data)
        return GetTaskId(**json_data)

    def mailbox_get_domains(self, **kwargs) -> Domains:
        json_data = super().mailbox_get_domains(**kwargs)
        catch_error(json_data)
        return Domains(**json_data)

    def mailbox_zones(self,
                      popular: int = "",
                      zones: int = "",
                      site: str = "",
                      **kwargs
                      ) -> MailboxZones:
        json_data = super().mailbox_zones(popular, zones, site, **kwargs)
        catch_error(json_data)
        return MailboxZones(json_data.get("status"), zones=[Zone(**x) for x in json_data.get("zones")] if json_data.get("zones") else None, popular=[Popular(**x) for x in json_data.get("popular")] if json_data.get("popular") else None)



class KopeechkaApiError(Exception):
    def __init__(self, status: str, value: str):
        self.status = status
        self.value = value

    def __str__(self):
        return self.value


class TimeOut(Exception):
    def __str__(self):
        return "Timed out"


def catch_error(data: dict):
    if data.get("status") == "ERROR":
        raise KopeechkaApiError(data.get("status"), data.get("value"))
